#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@auteur: David ALBERTO  (www.astrolabe-science.fr)
Calcule l'exces de masse en s'appuyant sur la formule semi-empirique de
Bethe-Weizsacker, et trace cette excès pour différents nucléides isobares.
"""
import matplotlib.pyplot as plt
import numpy as np
import matplotlib.ticker as tick # mise en forme des étiquettes de graduations
from mendeleev.fetch import fetch_table

plt.rcParams["font.family"] = "Roboto"

A = 62 # nombre de masse
Zmin, Zmax = 25, 32

if A<10:
    str_A = '0' + str(A)
else:
    str_A = str(A)
NomFichier = 'stabiliteNucleides' + str_A

c_pair='forestgreen'
c_impair='crimson'

# paramètres du modèle :----------------------------
a_v=15.56
a_s=17.23
a_c=0.697
a_a=23.285
a_p=12.0

m_e=0.511# masse électron MeV
m_n=939.5656# masse neutron MeV
m_p=938.2723# masse proton MeV
# --------------------------------------------------
symboles = [np.nan,'H','He','Li','Be','B','C','N','O','F','Ne','Na','Mg','Al','Si','P','S','Cl','Ar','K','Ca','Sc','Ti','V','Cr','Mn','Fe','Co','Ni','Cu','Zn','Ga','Ge','As','Se','Br','Kr','Rb','Sr','Y','Zr','Nb','Mo','Tc','Ru','Rh','Pd','Ag','Cd','In','Sn','Sb','Te','I','Xe','Cs','Ba','La','Ce','Pr','Nd','Pm','Sm','Eu','Gd','Tb','Dy','Ho','Er','Tm','Yb','Lu','Hf','Ta','W','Re','Os','Ir','Pt','Au','Hg','Tl','Pb','Bi','Po','At','Rn','Fr','Ra','Ac','Th','Pa','U','Np','Pu','Am','Cm','Bk','Cf','Es','Fm','Md','No','Lr','Rf','Db','Sg','Bh','Hs','Mt','Ds','Rg','Cn','Nh','Fl','Mc','Lv','Ts','Og']



# chargement des données sur les isotopes :
ptable = fetch_table('elements')
isotopes = fetch_table('isotopes', index_col='id')
colonnes=['atomic_number','mass_number','is_radioactive']
donnees=isotopes[colonnes]

def stable(Z,A):
    """
    Renvoie True si le nucléide (Z,A) est stable, False sinon.
    """
    radio = donnees.loc[(donnees['atomic_number']==Z) & (donnees['mass_number'] == A),'is_radioactive']
    return not radio.values[0]


def E_liaison(Z,A):
    """
    Calcule l'énergie de liaison en MeV.
    """
    if A%2!=0:# si A impair
        E_L=a_v*A-a_s*A**(2/3)-a_c*Z**2/A**(1/3)-a_a*(A-2*Z)**2/A
    else: #A pair
        if Z%2==0: # Z pair
            E_L=a_v*A-a_s*A**(2/3)-a_c*Z**2/A**(1/3)-a_a*(A-2*Z)**2/A+a_p/(A**(1/2))
        else: # Z impair
            E_L=a_v*A-a_s*A**(2/3)-a_c*Z**2/A**(1/3)-a_a*(A-2*Z)**2/A-a_p/(A**(1/2))
    return E_L

def masse_atomique(Z,A):
    """
    calcule la masse atomique en MeV
    """
    return (Z*(m_p+m_e)+(A-Z)*m_n) - E_liaison(Z, A)

def exces_masse(Z,A):
    """
    renvoie l'excès de masse en MeV
    """
    return masse_atomique(Z, A) - A*931.4943

exces_masse = np.vectorize(exces_masse)

def symbole(Z,pos):
    """
    renvoie une chaine contenant le n° atomique et le symbole d'un élément chimique.
    Pour la mise en forme des xlabels.
    """
    S = symboles[Z]
    return f'{Z}/{S}'

def minimum(liste_X,liste_Y):
    """
    renvoie les coordonnées du minimum Y de la liste
    """
    Ymin=liste_Y[0]
    for Y in liste_Y:
        if Y<Ymin:
            Ymin=Y
    N=list(liste_Y).index(Ymin)# rang du minimum
    X=liste_X[N]
    return X, Ymin


def paraboles(A):
    """
    pour A pair, trace les paraboles qui encadrent la courbe.
    """
    Z_continu = np.arange(Zmin, Zmax, 0.2)
    E_L_supp=a_v*A-a_s*A**(2/3)-a_c*Z_continu**2/A**(1/3)-a_a*(A-2*Z_continu)**2/A - a_p/(A**(1/2))
    exces_supp=Z_continu*(m_p+m_e)+(A-Z_continu)*m_n - E_L_supp - A*931.4943
    ax.plot(Z_continu,exces_supp,'--',lw=1,c='gray')
    E_L_inf=a_v*A-a_s*A**(2/3)-a_c*Z_continu**2/A**(1/3)-a_a*(A-2*Z_continu)**2/A + a_p/(A**(1/2))
    exces_inf=Z_continu*(m_p+m_e)+(A-Z_continu)*m_n - E_L_inf - A*931.4943
    ax.plot(Z_continu,exces_inf,'--',lw=1,c='gray')    


def affiche_stables():
    """
    Affiche un point vert pour le(s) n° atomique(s) du(des) nucléide(s) stable(s).
    """
    ymin, ymax = ax.get_ylim()
    for i in range(Zmin,Zmax+1):
        if stable(i,A):
            ax.scatter(i,ymin,s=40,c='yellowgreen', ec='forestgreen', clip_on=False, zorder=3)
    ax.set_ylim(ymin,ymax)
# -------------------------------------------------------------

fig, ax=plt.subplots(tight_layout=True)
fig.set_size_inches(15/2.54, 21/2.54)

ax.set_ylabel("excès de masse (MeV)")
ax.xaxis.set_major_formatter(tick.FuncFormatter(symbole))
Z = np.arange(Zmin,Zmax+1)
ax.set_xticks(Z)
if A%2==0:
    couleur=c_pair
    paraboles(A)
else:
    couleur=c_impair
# ax.tick_params(axis='x', labelsize = 9)

Y=exces_masse(Z,A)

ax.plot(Z,Y,'-o')
ax.text(0.5,0.5,f'A = {A:.0f}',c=couleur,transform=ax.transAxes,fontsize=16,ha='center')

# point pour le minimum d'excès de masse :
ax.scatter(minimum(Z,Y)[0],minimum(Z,Y)[1],c='orange',ec='k',zorder=3)

affiche_stables()



# fig.savefig(NomFichier + ".pdf")
fig.savefig(NomFichier + ".png", dpi=150)
